package gregapi.item.multiitem;

import static gregapi.data.CS.*;
import enviromine.handlers.EM_StatusManager;
import enviromine.trackers.EnviroDataTracker;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.IItemContainer;
import gregapi.code.TagData;
import gregapi.cover.CoverRegistry;
import gregapi.cover.ICover;
import gregapi.data.CS.ModIDs;
import gregapi.data.*;
import gregapi.data.TC.TC_AspectStack;
import gregapi.item.IItemEnergy;
import gregapi.item.multiitem.behaviors.IBehavior;
import gregapi.item.multiitem.food.IFoodStat;
import gregapi.item.prefixitem.PrefixItem;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictManager;
import gregapi.util.OM;
import gregapi.util.UT;

import java.util.Arrays;
import java.util.BitSet;
import java.util.HashMap;
import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumAction;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidContainerRegistry;
import net.minecraftforge.fluids.FluidContainerRegistry.FluidContainerData;
import net.minecraftforge.fluids.FluidStack;
import cpw.mods.fml.common.Optional;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 * 
 * For Custom Items.
 */
@Optional.InterfaceList(value = {
  @Optional.Interface(iface = "squeek.applecore.api.food.IEdible", modid = ModIDs.APC),
  @Optional.Interface(iface = "ic2.api.item.IItemReactorPlanStorage", modid = ModIDs.IC2C)
})
public abstract class MultiItemRandom extends MultiItem implements squeek.applecore.api.food.IEdible, ic2.api.item.IItemReactorPlanStorage, Runnable {
	public final BitSet mEnabledItems = new BitSet(32767);
	public final BitSet mVisibleItems = new BitSet(32767);
	public final IIcon[][] mIconList = new IIcon[32767][1];
	
	public final HashMap<Short, IFoodStat> mFoodStats = new HashMap<Short, IFoodStat>();
	public final HashMap<Short, IItemEnergy> mElectricStats = new HashMap<Short, IItemEnergy>();
	public final HashMap<Short, Short> mBurnValues = new HashMap<Short, Short>();
	
	/**
	 * Creates the Item using these Parameters.
	 * @param aUnlocalized The unlocalised Name of this Item. DO NOT START YOUR UNLOCALISED NAME WITH "gt."!!!
	 */
	public MultiItemRandom(String aModID, String aUnlocalized) {
		super(aModID, aUnlocalized);
		// Execute after all the other things. This is to ensure that MultiItems are created after PrefixItems.
		GAPI.mBeforeInit.add(this);
	}
	
	/**
	 * Add your Items here, and not within the Constructor.
	 * This gets called after all the PrefixItems and PrefixBlocks have been registered to the OreDict, what is during the @Init-Phase of the regular API.
	 */
	public abstract void addItems();
	
	private boolean mAllowedToAddItems = F;
	
	@Override
	public final void run() {
		mAllowedToAddItems = T;
		addItems();
	}
	
	@Override
	public ItemStack onItemRightClick(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		use(aStack, 0, aPlayer);
		isItemStackUsable(aStack);
		IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
		if (tStat != null && (UT.Entities.isCreative(aPlayer) || aPlayer.getFoodStats().needFood() || tStat.alwaysEdible(this, aStack, aPlayer))) aPlayer.setItemInUse(aStack, getMaxItemUseDuration(aStack));
		return super.onItemRightClick(aStack, aWorld, aPlayer);
	}
	
	/**
	 * This adds a Custom Item.
	 * @param aID The Id of the assigned Item [0 - 32766]
	 * @param aEnglish The Default Localised Name of the created Item
	 * @param aToolTip The Default ToolTip of the created Item, you can also insert null for having no ToolTip
	 * @param aFoodBehavior The Food Value of this Item. Can be null aswell. Just a convenience thing.
	 * @param aRandomData The OreDict Names you want to give the Item. Also used for TC Aspects and some other things.
	 * @return An ItemStack containing the newly created Item.
	 */
	public final ItemStack addItem(int aID, String aEnglish, String aToolTip, Object... aRandomData) {
		if (aToolTip == null) aToolTip = "";
		if (mAllowedToAddItems && aID >= 0 && aID < 32767 && aID != W) {
			ItemStack rStack = UT.Stacks.make(this, 1, aID);
			mEnabledItems.set(aID);
			mVisibleItems.set(aID);
			LH.add(getUnlocalizedName(rStack) + ".name", aEnglish);
			LH.add(getUnlocalizedName(rStack) + ".tooltip", aToolTip);
			List<TC_AspectStack> tAspects = new ArrayListNoNulls<TC_AspectStack>();
			// Important Stuff to do first
			for (Object tRandomData : aRandomData) if (tRandomData instanceof TagData) {
				if (tRandomData == TD.Creative.HIDDEN) {
					mVisibleItems.set(aID, F);
					continue;
				}
				if (tRandomData == TD.Properties.AUTO_BLACKLIST) {
					OreDictManager.INSTANCE.addToBlacklist_(rStack);
					continue;
				}
			}
			// now check for the rest
			for (Object tRandomData : aRandomData) if (tRandomData != null) {
				boolean tUseOreDict = T;
				if (tRandomData instanceof TagData) {
					continue;
				}
				if (tRandomData instanceof IFoodStat) {
					setFoodBehavior(aID, (IFoodStat)tRandomData);
					if (getContainerItem(rStack) == null) {
						int tFoodValue = ((IFoodStat)tRandomData).getFoodLevel(this, rStack, null);
						if (tFoodValue > 0) RM.Canner.addRecipe2(T, 16, tFoodValue * 16, rStack, IL.IC2_Food_Can_Empty.get(tFoodValue), ((IFoodStat)tRandomData).isRotten(this, rStack, null)?IL.IC2_Food_Can_Spoiled.get(tFoodValue):IL.IC2_Food_Can_Filled.get(tFoodValue));
					}
					tUseOreDict = F;
				}
				if (tRandomData instanceof ICover) {
					CoverRegistry.put(rStack, (ICover)tRandomData);
					tUseOreDict = F;
				}
				if (tRandomData instanceof IBehavior) {
					addItemBehavior(aID, (IBehavior<MultiItem>)tRandomData);
					tUseOreDict = F;
				}
				if (tRandomData instanceof IItemEnergy) {
					setElectricStats(aID, (IItemEnergy)tRandomData);
					tUseOreDict = F;
				}
				if (tRandomData instanceof IItemContainer) {
					((IItemContainer)tRandomData).set(rStack);
					tUseOreDict = F;
				}
				if (tRandomData instanceof TC_AspectStack) {
					((TC_AspectStack)tRandomData).addToAspectList(tAspects);
					continue;
				}
				if (tRandomData instanceof OreDictItemData) {
					if (((OreDictItemData)tRandomData).hasValidPrefixMaterialData()) {
						ItemStack tStack = ((OreDictItemData)tRandomData).getStack(1);
						// Priority over autogenerated PrefixItems, but not over the hardcoded Compatibility Targets.
						if (UT.Stacks.invalid(tStack) || tStack.getItem() instanceof PrefixItem) {
							OreDictManager.INSTANCE.setTarget_(((OreDictItemData)tRandomData).mPrefix, ((OreDictItemData)tRandomData).mMaterial.mMaterial, rStack);
							continue;
						}
					}
					OreDictManager.INSTANCE.addItemData_(rStack, (OreDictItemData)tRandomData);
					continue;
				}
				if (tRandomData instanceof FluidStack) {
					tRandomData = new FluidContainerData((FluidStack)tRandomData, UT.Stacks.copy(rStack), getContainerItem(rStack), T);
					if (((FluidContainerData)tRandomData).emptyContainer != null)
					RM.Canner.addRecipe1(T, 16, Math.max(((FluidContainerData)tRandomData).fluid.amount / 64, 16), ((FluidContainerData)tRandomData).emptyContainer, ((FluidContainerData)tRandomData).fluid, NF, ((FluidContainerData)tRandomData).filledContainer);
					RM.Canner.addRecipe1(T, 16, Math.max(((FluidContainerData)tRandomData).fluid.amount / 64, 16), ((FluidContainerData)tRandomData).filledContainer, NF, ((FluidContainerData)tRandomData).fluid, UT.Stacks.container(((FluidContainerData)tRandomData).filledContainer, F));
					UT.Fluids.setFluidContainerData((FluidContainerData)tRandomData, T);
					FluidContainerRegistry.registerFluidContainer((FluidContainerData)tRandomData);
					continue;
				}
				if (tRandomData instanceof FluidContainerData) {
					if (((FluidContainerData)tRandomData).emptyContainer != null)
					RM.Canner.addRecipe1(T, 16, Math.max(((FluidContainerData)tRandomData).fluid.amount / 64, 16), ((FluidContainerData)tRandomData).emptyContainer, ((FluidContainerData)tRandomData).fluid, NF, ((FluidContainerData)tRandomData).filledContainer);
					RM.Canner.addRecipe1(T, 16, Math.max(((FluidContainerData)tRandomData).fluid.amount / 64, 16), ((FluidContainerData)tRandomData).filledContainer, NF, ((FluidContainerData)tRandomData).fluid, UT.Stacks.container(((FluidContainerData)tRandomData).filledContainer, F));
					UT.Fluids.setFluidContainerData((FluidContainerData)tRandomData, T);
					FluidContainerRegistry.registerFluidContainer((FluidContainerData)tRandomData);
					continue;
				}
				if (tRandomData instanceof Runnable) {
					GAPI.mAfterPostInit.add((Runnable)tRandomData);
					continue;
				}
				if (tUseOreDict) {
					OM.reg(tRandomData, rStack);
					continue;
				}
			}
			if (GT_API.sCompatTC != null) GT_API.sCompatTC.registerThaumcraftAspectsToItem(rStack, tAspects, F);
			
			rStack = UT.Stacks.make(this, 1, aID);
			UT.Stacks.update(rStack);
			return rStack;
		}
		return null;
	}
	
	/**
	 * Sets a Food Behavior for the Item.
	 * 
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32766]
	 * @param aFoodBehavior the Food Behavior you want to add.
	 * @return the Item itself for convenience in constructing.
	 */
	public MultiItemRandom setFoodBehavior(int aMetaValue, IFoodStat aFoodBehavior) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length()) return this;
		if (aFoodBehavior == null) mFoodStats.remove((short)aMetaValue); else mFoodStats.put((short)aMetaValue, aFoodBehavior);
		return this;
	}
	
	/**
	 * Sets the Furnace Burn Value for the Item.
	 * 
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32766]
	 * @param aValue 200 = 1 Burn Process = 500 EU, max = 32767 (that is 81917.5 EU)
	 * @return the Item itself for convenience in constructing.
	 */
	public MultiItemRandom setBurnValue(int aMetaValue, int aValue) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length() || aValue < 0) return this;
		if (aValue == 0) mBurnValues.remove((short)aMetaValue); else mBurnValues.put((short)aMetaValue, aValue>Short.MAX_VALUE?Short.MAX_VALUE:(short)aValue);
		return this;
	}
	
	/**
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32766]
	 * @return the Item itself for convenience in constructing.
	 */
	public MultiItemRandom setElectricStats(int aMetaValue, IItemEnergy aStats) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length()) return this;
		if (aStats == null) mElectricStats.remove((short)aMetaValue); else {
			mElectricStats.put((short)aMetaValue, aStats);
			mIconList[aMetaValue] = Arrays.copyOf(mIconList[aMetaValue], Math.max(9, mIconList[aMetaValue].length));
		}
		return this;
	}
	
	/**
	 * @param aMetaValue the Meta Value of the Item you want to set it to. [0 - 32766]
	 * @param aMaxCharge Maximum Charge. (if this is == 0 it will remove the Electric Behavior)
	 * @param aTransferLimit Transfer Limit.
	 * @param aTier The electric Tier.
	 * @param aSpecialData If this Item has a Fixed Charge, like a SingleUse Battery (if > 0).
	 * Use -1 if you want to make this Battery chargeable (the use and canUse Functions will still discharge if you just use this)
	 * Use -2 if you want to make this Battery dischargeable.
	 * Use -3 if you want to make this Battery charge/discharge-able.
	 * @return the Item itself for convenience in constructing.
	 */
	public MultiItemRandom setFluidContainerStats(int aMetaValue, long aCapacity, long aStacksize) {
		if (aMetaValue < 0 || aMetaValue >= mEnabledItems.length()) return this;
		if (aCapacity < 0) mFluidContainerStats.remove((short)aMetaValue); else mFluidContainerStats.put((short)aMetaValue, new Long[] {aCapacity, Math.max(1, aStacksize)});
		return this;
	}
	
	@Override
	public int getMaxItemUseDuration(ItemStack aStack) {
		IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
		return tStat == null ? 0 : Math.max(tStat.getFoodLevel(this, aStack, null) * 8, 16);
	}
	
	@Override
	public EnumAction getItemUseAction(ItemStack aStack) {
		IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
		return tStat == null ? EnumAction.none : tStat.getFoodAction(this, aStack);
	}
	
	@Override
	public ItemStack onEaten(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
		if (tStat != null) {
			if (tStat.useAppleCoreFunctionality(this, aStack, aPlayer) && MD.APC.mLoaded) {
				aPlayer.getFoodStats().func_151686_a((ItemFood)UT.Reflection.callConstructor("squeek.applecore.api.food.ItemFoodProxy", 0, null, T, this), aStack);
			} else {
				aPlayer.getFoodStats().addStats(tStat.getFoodLevel(this, aStack, aPlayer), tStat.getSaturation(this, aStack, aPlayer));
			}
			if (!aWorld.isRemote && MD.ENVM.mLoaded) {
				try {
					float tTemperature = tStat.getTemperature(this, aStack, aPlayer) - C, tHydration = tStat.getHydration(this, aStack, aPlayer);
					Object tTracker = EM_StatusManager.lookupTracker(aPlayer);
					if (tTracker != null && ((EnviroDataTracker)tTracker).bodyTemp >= 0) {
						((EnviroDataTracker)tTracker).bodyTemp += (tTemperature - ((EnviroDataTracker)tTracker).bodyTemp) * tStat.getTemperatureEffect(this, aStack, aPlayer);
						if (tHydration > 0) ((EnviroDataTracker)tTracker).hydrate(tHydration); else if (tHydration < 0) ((EnviroDataTracker)tTracker).dehydrate(-tHydration);
					}
				} catch(Throwable e) {
					e.printStackTrace(ERR);
				}
			}
			tStat.onEaten(this, aStack, aPlayer, T);
		}
		return aStack;
	}
	
	@Override
	public IItemEnergy getEnergyStats(ItemStack aStack) {
		return mElectricStats.get(UT.Stacks.meta(aStack));
	}
	
	@Override
	public Long[] getFluidContainerStats(ItemStack aStack) {
		return mFluidContainerStats.get(UT.Stacks.meta(aStack));
	}
	
	@Override
	@Optional.Method(modid = ModIDs.APC)
	public squeek.applecore.api.food.FoodValues getFoodValues(ItemStack aStack) {
		IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
		return tStat==null?null:new squeek.applecore.api.food.FoodValues(tStat.getFoodLevel(this, aStack, null), tStat.getSaturation(this, aStack, null));
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubItems(Item var1, CreativeTabs aCreativeTab, List aList) {
		for (int i = 0, j = mEnabledItems.length(); i < j; i++) if (mVisibleItems.get(i) || (SHOW_HIDDEN_ITEMS && mEnabledItems.get(i))) {
			IItemEnergy tStats = mElectricStats.get((short)i);
			if (tStats == null) {
				ItemStack tStack = UT.Stacks.make(this, 1, i);
				isItemStackUsable(tStack);
				aList.add(tStack);
			} else {
				ItemStack
				tStack = UT.Stacks.make(this, 1, i);
				isItemStackUsable(tStack);
				aList.add(tStack);
				tStack = UT.Stacks.make(this, 1, i);
				for (TagData tEnergyType : tStats.getEnergyTypes(tStack)) tStats.setEnergyStored(tEnergyType, tStack, Long.MAX_VALUE);
				isItemStackUsable(tStack);
				aList.add(tStack);
			}
		}
		if (aList.isEmpty()) UT.Stacks.hide(this);
	}
	
	@Override
	@SideOnly(Side.CLIENT)
	public void registerIcons(IIconRegister aIconRegister) {
		for (short i = 0, j = (short)mEnabledItems.length(); i < j; i++) if (mEnabledItems.get(i)) {
			for (byte k = 1; k < mIconList[i].length; k++) {
				mIconList[i][k] = aIconRegister.registerIcon(mModID + ":" + getUnlocalizedName() + "/" + i + "/" + k);
			}
			mIconList[i][0] = aIconRegister.registerIcon(mModID + ":" + getUnlocalizedName() + "/" + i);
		}
	}
	
	@Override
	public IIcon getIconIndex(ItemStack aStack) {
		short aMetaData = UT.Stacks.meta(aStack);
		if (!UT.Code.exists(aMetaData, mIconList)) return null;
		IItemEnergy tStats = mElectricStats.get(aMetaData);
		if (tStats != null) {
			TagData tEnergyType = tStats.getEnergyTypes(aStack).iterator().next();
			long tStored = tStats.getEnergyStored(tEnergyType, aStack), tCapacity = tStats.getEnergyCapacity(tEnergyType, aStack);
			if (tStored <= 0) return mIconList[aMetaData][1];
			if (tStored >= tCapacity) return mIconList[aMetaData][8];
			return mIconList[aMetaData][7-(int)Math.max(0, Math.min(5, ((tCapacity-tStored)*6L) / tCapacity))];
		}
		return mIconList[aMetaData][0];
	}
	
	@Override
	public IIcon getIconFromDamage(int aMetaData) {
		return UT.Code.exists(aMetaData, mIconList) ? mIconList[aMetaData][0] : null;
	}
	
	@Override
	public IIcon getIcon(ItemStack aStack, int aRenderPass) {
		return getIconIndex(aStack);
	}
	
	@Override
	public IIcon getIcon(ItemStack aStack, int aRenderPass, EntityPlayer aPlayer, ItemStack aUsedStack, int aUseRemaining) {
		return getIconIndex(aStack);
	}
	
	@Override
	public IIcon getIconFromDamageForRenderPass(int aMetaData, int aRenderPass) {
		return getIconFromDamage(aMetaData);
	}
	
	@Override
	public void addAdditionalToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		IFoodStat tStat = mFoodStats.get((short)getDamage(aStack));
		if (tStat != null) tStat.addAdditionalToolTips(this, aList, aStack, aF3_H);
	}
	
	@Override
	public boolean isPlanStorage(ItemStack aStack) {
		return OM.is(OD_USB_STICKS[2], aStack);
	}
	
	@Override
	public boolean setSetup(ItemStack aStack, String aSetup) {
		if (OM.is(OD_USB_STICKS[2], aStack)) {
			if (!aStack.hasTagCompound()) aStack.setTagCompound(new NBTTagCompound());
			aStack.getTagCompound().setTag(NBT_USB_DATA, UT.NBT.getNBTString(UT.NBT.getNBTString(null, NBT_REACTOR_SETUP_NAME, ""+aSetup.hashCode()), NBT_REACTOR_SETUP, aSetup));
			aStack.getTagCompound().setByte(NBT_USB_TIER, (byte)2);
			return T;
		}
		return F;
	}
	
	@Override
	public void setPlanName(ItemStack aStack, String aName) {
		aStack.getTagCompound().getCompoundTag(NBT_USB_DATA).setString(NBT_REACTOR_SETUP_NAME, aName);
	}
	
	@Override
	public boolean hasSetup(ItemStack aStack) {
		return OM.is(OD_USB_STICKS[2], aStack) && aStack.hasTagCompound() && aStack.getTagCompound().getCompoundTag(NBT_USB_DATA).hasKey(NBT_REACTOR_SETUP);
	}
	
	@Override
	public String getSetup(ItemStack aStack) {
		return aStack.getTagCompound().getCompoundTag(NBT_USB_DATA).getString(NBT_REACTOR_SETUP);
	}
}