package gregapi.tileentity.energy;

import static gregapi.data.CS.*;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.TagData;
import gregapi.data.LH;
import gregapi.data.TD;
import gregapi.tileentity.machines.ITileEntitySwitchableOnOff;
import gregapi.util.UT;

import java.util.Collection;
import java.util.List;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;

public abstract class TileEntityBase11Twotypes extends TileEntityBase10EnergyConverter implements ITileEntitySwitchableOnOff {
	protected TagData mEnergyTypeEmitTwo = TD.Energy.QU;
	
    @Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
        if (aNBT.hasKey(NBT_ENERGY_EMITTED_2)) {
        	mEnergyTypeEmitTwo = TagData.createTagData(aNBT.getString(NBT_ENERGY_EMITTED_2));
        } else {
        	mEnergyTypeEmitTwo = mEnergyTypeEmitted;
        }
    }
    
	@Override
	public void addToolTipsEnergy(List aList, ItemStack aStack, boolean aF3_H) {
		LH.addEnergyToolTips(this, aList, mEnergyTypeAccepted, mEnergyTypeEmitted, mEnergyTypeEmitTwo, getLocalisedInputSide(), getLocalisedOutputSide(), getLocalisedOutputSide2());
	}
	
	@Override
	public void addToolTipsEfficiency(List aList, ItemStack aStack, boolean aF3_H) {
		if (TD.Energy.ALL_EU.contains(mEnergyTypeAccepted)) {
			if (TD.Energy.ALL_EU.contains(mEnergyTypeEmitted)) {
				aList.add(LH.getToolTipEfficiency(UT.Code.units(10000, mInput, mOutput, F)));
			} else {
				if (mEnergyTypeEmitted == TD.Energy.RF) aList.add(LH.getToolTipEfficiency(UT.Code.units(10000, mInput*4, mOutput, F)));
			}
			if (TD.Energy.ALL_EU.contains(mEnergyTypeEmitTwo)) {
				aList.add(LH.getToolTipEfficiency(UT.Code.units(10000, mInput, mOutput, F)));
			} else {
				if (mEnergyTypeEmitTwo == TD.Energy.RF) aList.add(LH.getToolTipEfficiency(UT.Code.units(10000, mInput*4, mOutput, F)));
			}
		} else {
			if (TD.Energy.ALL_EU.contains(mEnergyTypeEmitted) && mEnergyTypeAccepted == TD.Energy.RF) aList.add(LH.getToolTipEfficiency(UT.Code.units(10000, mInput, mOutput*8, F)));
			if (TD.Energy.ALL_EU.contains(mEnergyTypeEmitTwo) && mEnergyTypeAccepted == TD.Energy.RF) aList.add(LH.getToolTipEfficiency(UT.Code.units(10000, mInput, mOutput*8, F)));
		}
	}
	
    @Override
    public void doConversion(long aTimer) {
    	long tOutput = UT.Code.units(mEnergy, mInput, mOutput, F), tMaxOut = getEnergySizeOutputMax(mEnergyTypeEmitted, SIDE_ANY);
    	mActive = (tOutput >= getEnergySizeOutputMin(mEnergyTypeEmitted, SIDE_ANY));
		mEmitsEnergy = F;
    	
    	if (mActive) {
			if (tOutput > tMaxOut && TD.Energy.ALL_COMSUMPTION_LIMITED.contains(mEnergyTypeAccepted)) tOutput = tMaxOut;
    		if (tOutput > tMaxOut) {
				overload(mEnergy, mEnergyTypeEmitted);
    		} else {
				if (TD.Energy.ALL_SIZE_IRRELEVANT.contains(mEnergyTypeEmitted)) {
					long
					tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToSide(mEnergyTypeEmitted,           mFacing , 1, tOutput*mMultiplier, this);
					if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets, mOutput*mMultiplier, mInput, T);
					mEmitsEnergy = (tEmittedPackets > 0);
					
					if (TD.Energy.ALL_SIZE_IRRELEVANT.contains(mEnergyTypeEmitTwo)) {
						tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToSide(mEnergyTypeEmitTwo, OPPOSITES[mFacing], 1, tOutput*mMultiplier, this);
						if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets, mOutput*mMultiplier, mInput, T);
						mEmitsEnergy = (tEmittedPackets > 0 || mEmitsEnergy);
					} else {
						tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToSide(mEnergyTypeEmitTwo, OPPOSITES[mFacing], tOutput, mMultiplier, this);
						if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets * tOutput, mOutput*mMultiplier, mInput, T);
						mEmitsEnergy = (tEmittedPackets > 0 || mEmitsEnergy);
					}
				} else {
					long
					tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToSide(mEnergyTypeEmitted,           mFacing , tOutput, mMultiplier, this);
					if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets * tOutput, mOutput*mMultiplier, mInput, T);
					mEmitsEnergy = (tEmittedPackets > 0);
					
					if (TD.Energy.ALL_SIZE_IRRELEVANT.contains(mEnergyTypeEmitTwo)) {
						tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToSide(mEnergyTypeEmitTwo, OPPOSITES[mFacing], 1, tOutput*mMultiplier, this);
						if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets, mOutput*mMultiplier, mInput, T);
						mEmitsEnergy = (tEmittedPackets > 0 || mEmitsEnergy);
					} else {
						tEmittedPackets = ITileEntityEnergy.Util.emitEnergyToSide(mEnergyTypeEmitTwo, OPPOSITES[mFacing], tOutput, mMultiplier, this);
						if (!mWasteEnergy) mEnergy -= UT.Code.units(tEmittedPackets * tOutput, mOutput*mMultiplier, mInput, T);
						mEmitsEnergy = (tEmittedPackets > 0 || mEmitsEnergy);
					}
				}
    		}
    	}
		if (mWasteEnergy) mEnergy = Math.max(0, mEnergy-getEnergySizeInputMax(mEnergyTypeAccepted, SIDE_ANY));
    }
    
	@Override public boolean isEnergyType					(TagData aEnergyType, byte aSide, boolean aEmitting) {return aEmitting ? aEnergyType == mEnergyTypeEmitted || aEnergyType == mEnergyTypeEmitTwo : aEnergyType == mEnergyTypeAccepted;}
	@Override public boolean isEnergyAcceptingFrom			(TagData aEnergyType, byte aSide, boolean aTheoretical) {return (aTheoretical || !mStopped) && (SIDES_INVALID[aSide] || isInput (aSide)) && super.isEnergyAcceptingFrom(aEnergyType, aSide, aTheoretical);}
	@Override public boolean isEnergyEmittingTo				(TagData aEnergyType, byte aSide, boolean aTheoretical) {return (SIDES_INVALID[aSide] || (aEnergyType == mEnergyTypeEmitted ? isOutput(aSide) : isOutput2(aSide))) && super.isEnergyEmittingTo(aEnergyType, aSide, aTheoretical);}
	@Override public Collection<TagData> getEnergyTypes(byte aSide) {return new ArrayListNoNulls<TagData>(F, mEnergyTypeAccepted, mEnergyTypeEmitted, mEnergyTypeEmitTwo);}
	
	@Override public boolean isInput  (byte aSide) {return !ALONG_AXIS[aSide][mFacing];}
	@Override public boolean isOutput (byte aSide) {return aSide == mFacing;}
			  public boolean isOutput2(byte aSide) {return aSide == OPPOSITES[mFacing];}
	@Override public String getLocalisedInputSide  () {return LH.get(LH.FACE_ANYBUT_FRONT_BACK);}
	@Override public String getLocalisedOutputSide () {return LH.get(LH.FACE_FRONT);}
			  public String getLocalisedOutputSide2() {return LH.get(LH.FACE_BACK);}
}