package gregapi.tileentity.tank;

import static gregapi.data.CS.*;
import enviromine.handlers.EM_StatusManager;
import enviromine.trackers.EnviroDataTracker;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_AddToolTips;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnItemRightClick;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnItemUseFirst;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnlyPlaceableWhenSneaking;
import gregapi.block.multitileentity.*;
import gregapi.data.CS.BlocksGT;
import gregapi.data.CS.DrinksGT;
import gregapi.data.CS.SFX;
import gregapi.data.*;
import gregapi.data.LH.Chat;
import gregapi.fluid.FluidTankGT;
import gregapi.item.multiitem.food.FoodStatFluid;
import gregapi.tileentity.ITileEntityConnectedTank;
import gregapi.tileentity.base.TileEntityBase07Paintable;
import gregapi.util.UT;
import ic2.core.crop.TileEntityCrop;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.EnumAction;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidBlock;
import net.minecraftforge.fluids.IFluidContainerItem;
import squeek.applecore.api.food.FoodValues;

/**
 * @author Gregorius Techneticies
 */
public abstract class TileEntityBase08FluidContainer extends TileEntityBase07Paintable implements ITileEntityConnectedTank, IMTE_OnlyPlaceableWhenSneaking, IMTE_OnItemRightClick, IMTE_OnItemUseFirst, IMTE_AddToolTips, IFluidContainerItem {
	public FluidTankGT mTank = new FluidTankGT(1000);
	public boolean mGasProof = F, mAcidProof = F, mPlasmaProof = F;
	
	@Override
	public void readFromNBT2(NBTTagCompound aNBT) {
		super.readFromNBT2(aNBT);
    	if (aNBT.hasKey(NBT_GASPROOF)) mGasProof = aNBT.getBoolean(NBT_GASPROOF);
    	if (aNBT.hasKey(NBT_ACIDPROOF)) mAcidProof = aNBT.getBoolean(NBT_ACIDPROOF);
    	if (aNBT.hasKey(NBT_PLASMAPROOF)) mPlasmaProof = aNBT.getBoolean(NBT_PLASMAPROOF);
		if (aNBT.hasKey(NBT_TANK_CAPACITY)) mTank.setCapacity(aNBT.getLong(NBT_TANK_CAPACITY));
		mTank.readFromNBT(aNBT, NBT_TANK);
	}
	
	@Override
	public void writeToNBT2(NBTTagCompound aNBT) {
		super.writeToNBT2(aNBT);
		mTank.writeToNBT(aNBT, NBT_TANK);
	}
	
	@Override
	public NBTTagCompound writeItemNBT2(NBTTagCompound aNBT) {
		mTank.writeToNBT(aNBT, NBT_TANK);
		return super.writeItemNBT2(aNBT);
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		if (mTank.getFluidAmount() > 0) {
			aList.add(mTank.getFluidAmount() + " L of " + UT.Fluids.name(mTank.getFluid(), T) + " (" + (UT.Fluids.gas(mTank.getFluid()) ? "Gaseous" : "Liquid") + "; Max: "+mTank.getCapacity()+" L)");
			if (mTank.getFluidAmount() >= 250 && isDrinkable()) FoodStatFluid.INSTANCE.addAdditionalToolTips(aStack.getItem(), aList, aStack, aF3_H);
		} else {
			aList.add(Chat.CYAN + "Can store up to " + mTank.getCapacity() + " L");
		}
		if (mGasProof		) aList.add(Chat.ORANGE + LH.get(LH.TOOLTIP_GASPROOF));
		if (mAcidProof		) aList.add(Chat.ORANGE + LH.get(LH.TOOLTIP_ACIDPROOF));
		if (mPlasmaProof	) aList.add(Chat.ORANGE + LH.get(LH.TOOLTIP_PLASMAPROOF));
	}
	
	@Override
	public long onToolClick2(String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSide, float aHitX, float aHitY, float aHitZ) {
		long rReturn = super.onToolClick2(aTool, aRemainingDurability, aQuality, aPlayer, aChatReturn, aPlayerInventory, aSneaking, aStack, aSide, aHitX, aHitY, aHitZ);
		if (rReturn > 0 || isClientSide()) return rReturn;
		if (aTool.equals(TOOL_magnifyingglass)) {
			if (aChatReturn != null) {
				FluidStack tFluid = mTank.getFluid();
				if (tFluid == null) {
					aChatReturn.add("Container is empty");
				} else {
					aChatReturn.add("Contains: " + tFluid.amount + " L of " + UT.Fluids.name(tFluid, T) + " (" + (UT.Fluids.gas(tFluid) ? "Gaseous" : "Liquid") + ")");
				}
			}
			return 1;
		}
		return 0;
	}
	
	@Override
	public boolean onBlockActivated3(EntityPlayer aPlayer, byte aSide, float aHitX, float aHitY, float aHitZ) {
		if (isClientSide()) return T;
		
		ItemStack aStack = aPlayer.getCurrentEquippedItem(), tStack = UT.Stacks.container(UT.Stacks.amount(1, aStack), T);
		FluidStack tFluid = UT.Fluids.getFluidForFilledItem(UT.Stacks.amount(1, aStack), T);
		if (aStack != null && isFluidAllowed(tFluid) && mTank.fillAll(tFluid)) {
			aStack.stackSize--;
			if (tStack != null && !UT.Inventories.addStackToPlayerInventory(aPlayer, tStack, T)) worldObj.spawnEntityInWorld(new EntityItem(worldObj, xCoord+0.5, yCoord+0.5, zCoord+0.5, tStack));
			return T;
		}
		if (aStack != null) if ((tStack = UT.Fluids.fillFluidContainer(mTank, UT.Stacks.amount(1, aStack), T, T, T)) != null) {
			aStack.stackSize--;
			if (!UT.Inventories.addStackToPlayerInventory(aPlayer, tStack, T)) worldObj.spawnEntityInWorld(new EntityItem(worldObj, xCoord+0.5, yCoord+0.5, zCoord+0.5, tStack));
			return T;
		}
		
		if (isDrinkable()) {
			aStack = toStack();
			if (aStack == null) return T;
			if (UT.Entities.isCreative(aPlayer) || aPlayer.getFoodStats().needFood() || FoodStatFluid.INSTANCE.alwaysEdible(aStack.getItem(), aStack, aPlayer)) {
				switch(FoodStatFluid.INSTANCE.getFoodAction(aStack.getItem(), aStack)) {
				case eat: UT.Sounds.send(worldObj, SFX.MC_EAT  , 1.0F, 1.0F, getCoords()); break;
				default : UT.Sounds.send(worldObj, SFX.MC_DRINK, 1.0F, 1.0F, getCoords()); break;
				}
				aStack.getItem().onEaten(aStack, worldObj, aPlayer);
				mTank.drain(250, T);
			}
		}
		return T;
	}
	
	@Override public boolean onlyPlaceableWhenSneaking() {return T;}
	@Override public boolean canDrop(int aInventorySlot) {return F;}
	
	@Override
	public FluidStack getFluid(ItemStack aStack) {
		return mTank.getFluid();
	}
	
	@Override
	public int getCapacity(ItemStack aStack) {
		return mTank.getCapacity();
	}
	
	@Override
	public int fill(ItemStack aStack, FluidStack aFluid, boolean aDoFill) {
		if (!isFluidAllowed(aFluid)) return 0;
		int tFilled = mTank.fill(aFluid, aDoFill);
		if (tFilled > 0 && aDoFill) UT.NBT.set(aStack, writeItemNBT(aStack.hasTagCompound() ? aStack.getTagCompound() : new NBTTagCompound()));
		return tFilled;
	}
	
	@Override
	public FluidStack drain(ItemStack aStack, int aMaxDrain, boolean aDoDrain) {
		FluidStack tDrained = mTank.drain(aMaxDrain, aDoDrain);
		if (tDrained != NF && aDoDrain) UT.NBT.set(aStack, writeItemNBT(aStack.hasTagCompound() ? aStack.getTagCompound() : new NBTTagCompound()));
		return tDrained;
	}
	
	@Override
	public boolean onItemUseFirst(MultiTileEntityItemInternal aItem, ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, byte aSide, float hitX, float hitY, float hitZ) {
		if (aWorld.isRemote || aPlayer == null || !aPlayer.canPlayerEdit(aX, aY, aZ, aSide, aStack)) return F;
		if (canWaterCrops()) {
			FluidStack mFluid = aItem.getFluid(aStack);
			if (UT.Fluids.water(mFluid)) {
				TileEntity tTileEntity = UT.Worlds.getTileEntity(aWorld, aX, aY, aZ, F);
				if (tTileEntity instanceof TileEntityCrop) {
					int tDrained = Math.min(200 - ((TileEntityCrop)tTileEntity).waterStorage, mFluid.amount * 10);
					if (tDrained > 0) {
						aItem.drain(aStack, (int)UT.Code.divup(tDrained, 10), T);
						((TileEntityCrop)tTileEntity).waterStorage += tDrained;
						UT.Sounds.send(aWorld, SFX.MC_LIQUID_WATER, 1.0F, 1.0F, aX, aY, aZ);
					}
					return T;
				}
			}
		}
		return F;
	}
	
	@Override
	public ItemStack onItemRightClick(MultiTileEntityItemInternal aItem, ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		if (isDrinkable() && (UT.Entities.isCreative(aPlayer) || aPlayer.getFoodStats().needFood() || FoodStatFluid.INSTANCE.alwaysEdible(aStack.getItem(), aStack, aPlayer))) {
			aPlayer.setItemInUse(aStack, Math.max(FoodStatFluid.INSTANCE.getFoodLevel(aStack.getItem(), aStack, null) * 8, 32));
			return aStack;
		}
		if (canPickUpFluids()) {
			MovingObjectPosition tPosition = UT.Worlds.getMovingObjectPositionFromPlayer(aWorld, aPlayer, T);
			if (tPosition == null || tPosition.typeOfHit != MovingObjectPosition.MovingObjectType.BLOCK) return aStack;
			if (!aWorld.canMineBlock(aPlayer, tPosition.blockX, tPosition.blockY, tPosition.blockZ)) return aStack;
			
			Block tBlock = aWorld.getBlock(tPosition.blockX, tPosition.blockY, tPosition.blockZ);
			if (tBlock == Blocks.water || tBlock == Blocks.flowing_water) {
				if (aWorld.getBlockMetadata(tPosition.blockX, tPosition.blockY, tPosition.blockZ) == 0 && aItem.fill(aStack, FL.Water.make(1000), F) == 1000) {
					aWorld.setBlockToAir(tPosition.blockX, tPosition.blockY, tPosition.blockZ);
					aItem.fill(aStack, FL.Water.make(1000), T);
				}
				return aStack;
			}
			if (tBlock == Blocks.lava || tBlock == Blocks.flowing_lava) {
				if (aWorld.getBlockMetadata(tPosition.blockX, tPosition.blockY, tPosition.blockZ) == 0 && aItem.fill(aStack, FL.Lava.make(1000), F) == 1000) {
					aWorld.setBlockToAir(tPosition.blockX, tPosition.blockY, tPosition.blockZ);
					aItem.fill(aStack, FL.Lava.make(1000), T);
				}
				return aStack;
			}
			if (tBlock == BlocksGT.Ocean) {
				aItem.fill(aStack, FL.Ocean.make(1000), T);
				return aStack;
			}
			if (tBlock instanceof IFluidBlock) {
				FluidStack tDrained = ((IFluidBlock)tBlock).drain(aWorld, tPosition.blockX, tPosition.blockY, tPosition.blockZ, F);
				if (tDrained != null && tDrained.amount > 0 && aItem.fill(aStack, tDrained, F) == tDrained.amount) {
					// Forge fucked up the Fluid Draining Function, meaning if you insert true for doDrain it will ALWAYS return a null Fluid for the finite Fluid Blocks. That's why I take the result from the simulation instead of the actual draining.
					aItem.fill(aStack, tDrained, T);
					((IFluidBlock)tBlock).drain(aWorld, tPosition.blockX, tPosition.blockY, tPosition.blockZ, T);
				}
				return aStack;
			}
			
			tPosition.blockX+=OFFSETS_X[tPosition.sideHit];
			tPosition.blockY+=OFFSETS_Y[tPosition.sideHit];
			tPosition.blockZ+=OFFSETS_Z[tPosition.sideHit];
			tBlock = aWorld.getBlock(tPosition.blockX, tPosition.blockY, tPosition.blockZ);
			
			if (tBlock instanceof IFluidBlock) {
				FluidStack tDrained = ((IFluidBlock)tBlock).drain(aWorld, tPosition.blockX, tPosition.blockY, tPosition.blockZ, F);
				if (tDrained != null && tDrained.amount > 0 && aItem.fill(aStack, tDrained, F) == tDrained.amount) {
					// Forge fucked up the Fluid Draining Function, meaning if you insert true for doDrain it will ALWAYS return a null Fluid for the finite Fluid Blocks. That's why I take the result from the simulation instead of the actual draining.
					aItem.fill(aStack, tDrained, T);
					((IFluidBlock)tBlock).drain(aWorld, tPosition.blockX, tPosition.blockY, tPosition.blockZ, T);
				}
				return aStack;
			}
		}
		return aStack;
	}
	
	public int getMaxItemUseDuration(MultiTileEntityItemInternal aItem, ItemStack aStack) {
		return isDrinkable() ? Math.max(FoodStatFluid.INSTANCE.getFoodLevel(aStack.getItem(), aStack, null) * 8, 32) : 0;
	}
	
	public EnumAction getItemUseAction(MultiTileEntityItemInternal aItem, ItemStack aStack) {
		return isDrinkable() ? FoodStatFluid.INSTANCE.getFoodAction(aStack.getItem(), aStack) : EnumAction.none;
	}
	
	public ItemStack onEaten(MultiTileEntityItemInternal aItem, ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		if (!isDrinkable()) return aStack;
		
		if (FoodStatFluid.INSTANCE.useAppleCoreFunctionality(aStack.getItem(), aStack, aPlayer) && MD.APC.mLoaded) {
			aPlayer.getFoodStats().func_151686_a((ItemFood)UT.Reflection.callConstructor("squeek.applecore.api.food.ItemFoodProxy", 0, null, T, aStack.getItem()), aStack);
		} else {
			aPlayer.getFoodStats().addStats(FoodStatFluid.INSTANCE.getFoodLevel(aStack.getItem(), aStack, aPlayer), FoodStatFluid.INSTANCE.getSaturation(aStack.getItem(), aStack, aPlayer));
		}
		if (!aWorld.isRemote && MD.ENVM.mLoaded) {
			try {
				float tTemperature = FoodStatFluid.INSTANCE.getTemperature(aStack.getItem(), aStack, aPlayer) - C, tHydration = FoodStatFluid.INSTANCE.getHydration(aStack.getItem(), aStack, aPlayer);
				Object tTracker = EM_StatusManager.lookupTracker(aPlayer);
				if (tTracker != null && ((EnviroDataTracker)tTracker).bodyTemp >= 0) {
					((EnviroDataTracker)tTracker).bodyTemp += (tTemperature - ((EnviroDataTracker)tTracker).bodyTemp) * FoodStatFluid.INSTANCE.getTemperatureEffect(aStack.getItem(), aStack, aPlayer);
					if (tHydration > 0) ((EnviroDataTracker)tTracker).hydrate(tHydration); else if (tHydration < 0) ((EnviroDataTracker)tTracker).dehydrate(-tHydration);
				}
			} catch(Throwable e) {
				e.printStackTrace(ERR);
			}
		}
		FoodStatFluid.INSTANCE.onEaten(aStack.getItem(), aStack, aPlayer, F);
		
		mTank.drain(250, T);
		UT.NBT.set(aStack, writeItemNBT(aStack.hasTagCompound() ? aStack.getTagCompound() : new NBTTagCompound()));
		return aStack;
	}
	
	public FoodValues getFoodValues(MultiTileEntityItemInternal aItem, ItemStack aStack) {
		return isDrinkable() ? new squeek.applecore.api.food.FoodValues(FoodStatFluid.INSTANCE.getFoodLevel(aStack.getItem(), aStack, null), FoodStatFluid.INSTANCE.getSaturation(aStack.getItem(), aStack, null)) : null;
	}
	
	@Override
	public int addFluidToConnectedTank(byte aSide, FluidStack aFluid, boolean aOnlyAddIfItAlreadyHasFluidsOfThatTypeOrIsDedicated) {
		if (aFluid == NF || (mTank.getFluidAmount() <= 0 && aOnlyAddIfItAlreadyHasFluidsOfThatTypeOrIsDedicated)) return 0;
		return mTank.fill(aFluid, T);
	}
	
	@Override
	public int removeFluidFromConnectedTank(byte aSide, FluidStack aFluid, boolean aOnlyRemoveIfItCanRemoveAllAtOnce) {
		if (aFluid == NF || !UT.Fluids.equal(mTank.getFluid(), aFluid) || mTank.getFluidAmount() < (aOnlyRemoveIfItCanRemoveAllAtOnce ? aFluid.amount : 1)) return 0;
		return mTank.drain(aFluid.amount, T).amount;
	}
	
	@Override
	public long getAmountOfFluidInConnectedTank(byte aSide, FluidStack aFluid) {
		return aFluid == NF || mTank.getFluid() == NF || !UT.Fluids.equal(mTank.getFluid(), aFluid) ? 0 : mTank.getFluidAmount();
	}
	
	public boolean isDrinkable() {
		return this instanceof IMTE_GetFoodValues && mTank.getFluidAmount() >= 250 && DrinksGT.REGISTER.containsKey(mTank.getFluid().getFluid().getName());
	}
	
	public boolean isFluidAllowed(FluidStack aFluid) {
		return aFluid != null && !UT.Fluids.anysteam(aFluid) && (mGasProof || !UT.Fluids.gas(aFluid)) && (mAcidProof || !UT.Fluids.acid(aFluid)) && (mPlasmaProof || !UT.Fluids.plasma(aFluid));
	}
	
	public boolean canWaterCrops() {
		return F;
	}
	
	public boolean canPickUpFluids() {
		return F;
	}
}